package org.oasis.wscaf.module.utils;

import java.util.HashMap;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;

public class CompensatorHandler {

	String classPrefix = "[CompensatorHandler]";
	
	private HashMap compensatorTable = new HashMap();
	
	public void addCompensator(String participantKey, Compensator compensator) {
		synchronized (compensatorTable) {
			compensatorTable.put(participantKey, compensator);
		}
	}
	
	public OMElement invoke(SOAPFactory factory, String participantKey, String operation) throws AxisFault {
		String opPrefix = classPrefix+"[invoke] ";
		
		System.out.println(opPrefix+"Executing "+operation+" over participant "+participantKey);
		Compensator compensator;
		OMElement res = null;
		synchronized (compensatorTable) {
			compensator = (Compensator) compensatorTable.get(participantKey);
		}
		if (compensator != null) {
			System.out.println("[CompensatorHandler] Compensator found");
			synchronized (compensator) {
				if (operation.equals(ContextModuleConstants.COMPENSATE_NAME)) {
					System.out.println(opPrefix + "It's a compensation");
					compensator.compensate(participantKey);
					res = factory.createOMElement(new QName(ContextModuleConstants.LRA_NS,"compensated"));
				}
				else if (operation.equals(ContextModuleConstants.COMPLETE_NAME)) {
					System.out.println(opPrefix + "It's a completion");
					compensator.complete(participantKey);
					res = factory.createOMElement(new QName(ContextModuleConstants.LRA_NS,"completed"));
				}
				else if (operation.equals(ContextModuleConstants.FORGET_NAME)) {
					System.out.println(opPrefix + "It's a forget");
					compensator.forget(participantKey);
					res = factory.createOMElement(new QName(ContextModuleConstants.LRA_NS,"forgot"));
				}
				else {
					System.out.println(opPrefix+"Operation "+operation+" is not recognized!!");
					throw new AxisFault("Operation "+operation+" is not a valid LRA action");
				}
			}
			if (operation.equals(ContextModuleConstants.COMPENSATE_NAME) || operation.equals(ContextModuleConstants.FORGET_NAME)) {
				synchronized (compensatorTable) {
					compensatorTable.remove(participantKey);
				}
			}
		}
		else {
			System.out
					.println("[CompensatorHandler] No compensator found for key "
							+ participantKey);
		}
		return res;
	}
}
