/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.module;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axiom.soap.impl.dom.soap12.SOAP12Factory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.receivers.RawXMLINOutMessageReceiver;
import org.oasis.wscaf.bindings.lra.LRAContextType;
import org.oasis.wscaf.module.utils.Compensator;
import org.oasis.wscaf.module.utils.ContextModuleConstants;

public class CompensationMessageReceiver extends RawXMLINOutMessageReceiver {
	

	String classPrefix = "[CompensationMessageReceiver]";
	@Override
	public void invokeBusinessLogic(MessageContext arg0, MessageContext arg1) throws AxisFault {
		String opPrefix = classPrefix+"[invokeBusinessLogic] ";
		LRAContextType ctx = (LRAContextType) arg0.getProperty(ContextModuleConstants.CurrentLRAContextProperty);
		
		if (ctx == null) {
			throw new AxisFault("No LRA context found in header.");
		}
		
		arg1.setProperty(ContextModuleConstants.OutLRAContextProperty, ctx);
		
		Compensator compensator = null;
		try {
			compensator = (Compensator) getTheImplementationObject(arg0);
		}
		catch (ClassCastException e) {
			throw new AxisFault("The invoked service is not compensable since it doesn't implement the Compensator interface.",e);
		}
		
		String responseName = null;
		if (compensator != null) {
			String action = arg0.getSoapAction();
			if (action != null) {
				if (action.equals(ContextModuleConstants.COMPENSATE_NAME)) {
					compensator.compensate(arg0.getEnvelope().getBody().getFirstChildWithName(new QName(ContextModuleConstants.LRA_NS,"compensate")));
					responseName = "compensated";
				}
				else if (action.equals(ContextModuleConstants.COMPLETE_NAME)) {
					compensator.complete(arg0.getEnvelope().getBody().getFirstChildWithName(new QName(ContextModuleConstants.LRA_NS,"complete")));
					responseName = "completed";
				}
				else if (action.equals(ContextModuleConstants.FORGET_NAME)) {
					compensator.forget(arg0.getEnvelope().getBody().getFirstChildWithName(new QName(ContextModuleConstants.LRA_NS,"forget")));
					responseName = "forgot";
				}
				else {
					throw new AxisFault("SOAPAction "+action+" is not recognized as a valid LRA operation");
				}
			}
			else {
				throw new AxisFault("SOAPAction is mandatory in order to execute compensation actions");
			}
			
		}
		else {
			throw new AxisFault("Could not find an implementation class for the invoked EPR");
		}
		
		SOAPFactory factory = getSOAPFactory(arg0);
		if (factory == null) {
			factory = new SOAP12Factory();
		}
		
		if (arg1.getEnvelope() == null) {
			SOAPEnvelope envelope = factory.getDefaultEnvelope();
			arg1.setEnvelope(envelope);
			OMElement res = factory.createOMElement(new QName(ContextModuleConstants.LRA_NS,responseName));
			arg1.getEnvelope().getBody().addChild(res);
		}
		
	}	

}
