/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.module.utils;

import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.AxisService;
import org.oasis.wscaf.bindings.wscf.RegistrationContextType;
import org.oasis.wscaf.bindings.wsctx.ContextType;

public class WSCAFInfo {

	private String contextServiceLocation = null;
	private String contextManagerLocation = null;
	private String registrationServiceLocation = null;
	
	public WSCAFInfo() {
		
	}
	
	public static WSCAFInfo getCurrentCafInfo() {
		MessageContext msgCtx = MessageContext.getCurrentMessageContext();
		if (msgCtx != null) {
			AxisService service = msgCtx.getAxisService();
			if (service != null) {
				return new WSCAFInfo(service);
			}
		}
		return null;
	}
	
	public WSCAFInfo(AxisService srvc) {
		this.contextServiceLocation = (String) srvc.getParameter(ContextModuleConstants.CONTEXT_SERVICE_LOCATION_PARAMETER_NAME).getValue();
		this.contextManagerLocation = (String) srvc.getParameter(ContextModuleConstants.CONTEXT_MANAGER_LOCATION_PARAMETER_NAME).getValue();
		this.registrationServiceLocation = (String) srvc.getParameter(ContextModuleConstants.REGISTRATION_SERVICE_LOCATION_PARAMETER_NAME).getValue();
	}
	
	public WSCAFInfo(String contextServiceLocation,
			String contextManagerLocation, String registrationServiceLocation) {
		super();
		this.contextServiceLocation = contextServiceLocation;
		this.contextManagerLocation = contextManagerLocation;
		this.registrationServiceLocation = registrationServiceLocation;
	}

	public String getContextServiceLocation() {
		return contextServiceLocation;
	}

	public void setContextServiceLocation(String contextServiceLocation) {
		this.contextServiceLocation = contextServiceLocation;
	}

	public String getContextManagerLocation() {
		return contextManagerLocation;
	}

	public void setContextManagerLocation(String contextManagerLocation) {
		this.contextManagerLocation = contextManagerLocation;
	}

	public String getRegistrationServiceLocation() {
		return registrationServiceLocation;
	}

	public void setRegistrationServiceLocation(String registrationServiceLocation) {
		this.registrationServiceLocation = registrationServiceLocation;
	}
	
	public void updateInfo(ContextType context) {
		try {
			this.contextServiceLocation = DatabindingHelper.serviceRefToEpr(context.getContextService()).getAddress();
			this.contextManagerLocation = DatabindingHelper.serviceRefToEpr(context.getContextManager()).getAddress();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void updateInfo(RegistrationContextType context) {
		updateInfo((ContextType)context);
		try {
			this.registrationServiceLocation = DatabindingHelper.serviceRefToEpr(context.getRegistrationService()).getAddress();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
}
