package org.oasis.wscaf.newcore.types.wsctx.impl;

import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Set;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.om.impl.llom.factory.OMLinkedListImplFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.addressing.EndpointReferenceHelper;
import org.oasis.wscaf.newcore.types.common.exceptions.MalformedElement;
import org.oasis.wscaf.newcore.types.wsctx.Context;
import org.oasis.wscaf.utils.AddressingUtils;

public class OMContext implements Context {
	
	protected HashMap augmenters = new HashMap();
	private EndpointReference contextManager;
	private EndpointReference contextService;
	private URI contextIdentifier;
	private Context parentContext;

	protected EndpointReference getReference(OMElement serviceRef) throws MalformedElement {
		OMElement eprElement = serviceRef.getFirstElement();
		if (eprElement != null) {
			try {
				EndpointReference epr = EndpointReferenceHelper.fromOM(eprElement);
				return epr;
			} catch (AxisFault e) {
				e.printStackTrace();
				throw new MalformedElement("Context service element:\n"+eprElement+"Is not a valid EndpointReference element.",e);
			}
		}
		else {
			throw new MalformedElement("Service reference:\n"+serviceRef+"Does not contain any child. A EndpointReference element is needed.");
		}
	}
	
	protected void putReference(OMElement parent, OMFactory factory, QName name, EndpointReference service) throws AxisFault {
		OMElement serviceRef = factory.createOMElement(name, parent);
		OMElement serviceElement = AddressingUtils.eprToOM(this.getContextManager());
		serviceRef.addChild(serviceElement);
	}
	
	public OMContext(URI contextIdentifier) {
		this.contextIdentifier = contextIdentifier;
	}

	public void addAugmenter(OMElement augmenter) {
		QName nodeName = augmenter.getQName();
		Object augmenterObject = augmenters.get(nodeName);
		if (augmenterObject != null) {
			ArrayList augmenterList = (ArrayList) augmenterObject;
			augmenterList.add(augmenter);
		}
		augmenters.put(nodeName, augmenter);
	}

	public ArrayList getAugmenter(QName name) {
		Object augmenter = augmenters.get(name);
		if (augmenter != null) {
			
			return (ArrayList) augmenter;
		}
		else {
			return null;
		}
	}

	public Set getAugmenters() {
		return augmenters.entrySet();
	}

	public EndpointReference getContextManager() {
		return contextManager;
	}

	public EndpointReference getContextService() {
		return contextService;
	}

	public URI getIdentifier() {
		return contextIdentifier;
	}

	public Context getParentContext() {
		return parentContext;
	}

	public void removeAugmenter(QName augmenter) {
		augmenters.remove(augmenter);
	}

	public void setContextManager(EndpointReference manager) {
		this.contextManager = manager;
	}

	public void setContextService(EndpointReference service) {
		this.contextService = service;
	}

	public void setIdentifier(URI identifier) {
		this.contextIdentifier = identifier;
	}

	public void setParentContext(Context parentContext) {
		this.parentContext = parentContext;
	}

	public void fromOM(OMElement element) throws MalformedElement {
		if (!element.getQName().equals(Context.NAME)) {
			throw new MalformedElement("Context element :\n"+element.toString()+"Does not start with "+Context.NAME+" element.");
		}
		Iterator childs = element.getChildElements();
		while (childs.hasNext()) {
			OMElement currentChild = (OMElement) childs.next();
			QName childName = currentChild.getQName();
			
			if (childName.equals(Context.CONTEXT_IDENTIFIER_NAME)) {
				try {
					URI contextURI = new URI(currentChild.getText());
					this.setIdentifier(contextURI);
				} catch (URISyntaxException e) {
					e.printStackTrace();
					throw new MalformedElement("Context identifier element:\n"+currentChild+"Does not contain a valid URI.",e);
				}
			}
			else if (childName.equals(Context.CONTEXT_SERVICE_NAME)) {
				EndpointReference service = getReference(currentChild);
				this.setContextService(service);	
			}
			else if (childName.equals(Context.CONTEXT_MANAGER_NAME)) {
				EndpointReference manager = getReference(currentChild);
				this.setContextManager(manager);
			}
			else {
				Object arrayObject = augmenters.get(childName);
				if (arrayObject != null) {
					ArrayList array = (ArrayList) arrayObject;
					array.add(currentChild);
				}
				else {
					ArrayList array = new ArrayList();
					array.add(currentChild);
					augmenters.put(childName, array);
				}
			}
		}
	}

	public QName getName() {
		return Context.NAME;
	}

	public OMElement toOM(OMFactory factory) {
		OMFactory realFactory = factory;
		if (factory == null) {
			realFactory = new OMLinkedListImplFactory();
		}
		OMElement context = realFactory.createOMElement(this.getName());
		
		OMElement contextId = realFactory.createOMElement(Context.CONTEXT_IDENTIFIER_NAME, context);
		contextId.setText(this.getIdentifier().toString());
		
		if (this.getContextManager() != null) {
			try {
				putReference(context, realFactory, Context.CONTEXT_MANAGER_NAME, this.getContextManager());
			} catch (AxisFault e) {
				System.out.println("Could not transform Context Manager"
						+ this.getContextManager() + " to OMElement");
				e.printStackTrace();
			}
		}
		
		if (this.getContextService() != null) {
			try {
				putReference(context, realFactory, Context.CONTEXT_SERVICE_NAME, this.getContextService());
			} catch (AxisFault e) {
				System.out.println("Could not transform Context Service"
						+ this.getContextService() + " to OMElement");
				e.printStackTrace();
			}
		}
		
		if (this.getParentContext() != null) {
			realFactory.createOMElement(Context.PARENT_CONTEXT_NAME, this.getParentContext().toOM(realFactory));
		}
		
		Set augmenterList = augmenters.entrySet();
		Iterator iterator = augmenterList.iterator();
		
		while (iterator.hasNext()) {
			ArrayList elementList = (ArrayList) iterator.next();
			for (int i=0; i<elementList.size();i++) {
				OMElement currentElement = (OMElement) elementList.get(i);
				context.addChild(currentElement);
			}
		}
		
		return context;
	}

}
