/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wscf.exceptions;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.soap.SOAPFaultDetail;
import org.apache.axiom.soap.impl.llom.soap11.SOAP11Factory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.AddressingConstants;
import org.apache.axis2.addressing.EndpointReferenceHelper;
import org.oasis.wscaf.core.wscf.types.Participant;
import org.oasis.wscaf.utils.Constants;

public class ParticipantNotFound extends AxisFault {

	private Participant requestedParticipant;
	
	public ParticipantNotFound(String cause, Participant requestedParticipant) {
		super(cause);
		this.requestedParticipant = requestedParticipant;
	}

	public ParticipantNotFound(Participant requestedParticipant) {
		super("Participant not found: " + requestedParticipant.toString());
		this.requestedParticipant = requestedParticipant;
	}

	public Participant getRequestedParticipant() {
		return requestedParticipant;
	}

	public QName getFaultCode() {
		return new QName(Constants.COORDINATION_NS,"ParticipantNotFound");
	}
	
	public SOAPFaultDetail getFaultDetailElement() {
		
		OMFactory factory = null;
		SOAPFaultDetail details = super.getFaultDetailElement(); 
		
		if (details == null) {
			factory = new SOAP11Factory();
			details = ((SOAP11Factory)factory).createSOAPFaultDetail();
		}
		else {
			factory = details.getOMFactory();
		}
		
		OMElement requestedElement = factory.createOMElement("requestedParticipant",factory.createOMNamespace(Constants.CONTEXT_NS,"ctx"));
		if (requestedParticipant != null) {
			try {
				requestedElement.addChild(EndpointReferenceHelper.toOM(factory, requestedParticipant.getEndPoint(), Constants.ENDPOINTREFERENCE_FINAL_NAME, AddressingConstants.Final.WSA_NAMESPACE));
			} catch (AxisFault e) {
				e.printStackTrace();
			}
		}
		
		details.addDetailEntry(requestedElement);
		
		return details;
	}

}
