/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wscf.types;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.apache.axis2.addressing.EndpointReference;
import org.oasis.wscaf.core.wscf.exceptions.DuplicateParticipant;
import org.oasis.wscaf.core.wscf.exceptions.ParticipantNotFound;

public class ParticipantList {
	private Map<String,ArrayList<Participant>> protocolMap;  // String ArrayLists {x..n} x==0 <=> participants.size() == 0
	//private ArrayList participants; // Participant  {0..n}
	
	public ParticipantList() {
		protocolMap  = Collections.synchronizedMap(new HashMap<String,ArrayList<Participant>>());
		//participants = new ArrayList(); 
	}
	
	public void addParticipant(Participant participant, String[] protocols) throws DuplicateParticipant {
		
		for (int i=0; i<protocols.length; i++) {
			String protocol=protocols[i];
			ArrayList<Participant> protocolParticipants;
			
			if (protocolMap.containsKey(protocol)) {
				protocolParticipants = protocolMap.get(protocol);
				if (!protocolParticipants.contains(participant))
					protocolParticipants.add(participant);
                else
                    throw new DuplicateParticipant("Participant " + participant.getEndPoint() + " already registered in protocol \"" + protocol + "\"", participant.getEndPoint());
			}
			else {
				protocolParticipants = new ArrayList<Participant>();
				protocolParticipants.add(participant);
				protocolMap.put(protocol, protocolParticipants);
			}
		}
	}
	
	public void removeParticipant(String protocol, Participant participant) throws ParticipantNotFound {
		ArrayList<Participant> protocolParticipants = protocolMap.get(protocol);
		if (protocol != null) {
			int participantIndex = protocolParticipants.indexOf(participant);
			if (participantIndex != -1) {
				protocolParticipants.remove(protocolParticipants.indexOf(participant));
			}
			else {
				throw new ParticipantNotFound(participant);
			}
		}
	}

	public void replaceParticipant(String protocol, Participant oldParticipant, Participant newParticipant) {
		ArrayList<Participant> protocolParticipants = protocolMap.get(protocol);
		
		if (protocol != null) {
			int participantIndex = protocolParticipants.indexOf(oldParticipant);
			if (participantIndex != -1) {
				protocolParticipants.get(participantIndex).copy(newParticipant);
			}
		}
	}
	
	public ParticipantGroup getParticipants(String[] protocols) {
		if (protocols==null || protocols.length<=0)
			return getAllParticipants();

		ParticipantGroup group = new ParticipantGroup();
		for (int i=0; i<protocols.length; i++) {
			String protocol = protocols[i];
			ArrayList<Participant> protocolList = protocolMap.get(protocol);
			if (protocolList!=null) {
				for (int j=0; j<protocolList.size(); j++)
					group.addParticipant(protocolList.get(j));
				group.addProtocol(protocol);
			}
		}
		return group;
	}
    
    private ParticipantGroup getAllParticipants() {
        ParticipantGroup group = new ParticipantGroup();
        
        Iterator<String> protocols = protocolMap.keySet().iterator();
        while (protocols.hasNext()) {
        	String currentProtocol = protocols.next();
            group.addProtocol(currentProtocol);
            ArrayList<Participant> participants = protocolMap.get(protocols.next());
            for (int i=0; i<participants.size(); i++) {
            	group.addParticipant((Participant)participants.get(i));
            }
        }
        return group;
    }

    public void print() {
        print("");
    }
    
	public void print(String header) {
		System.out.println(header + "---- Participant List ----");
		/*System.out.println(header + "Participants: ");
		for (int i=0; i<participants.size(); i++)
			System.out.println(header + "   Participant " + i + ": " + (Participant)participants.get(i));
        System.out.println(header + "Protocols: ");*/
		
		Iterator<String> protocols = protocolMap.keySet().iterator();
		while (protocols.hasNext()) {
			String protocol = (String)protocols.next();
			ArrayList<Participant> protocolList = protocolMap.get(protocol);
			System.out.println(header + "   Protocol \"" + protocol + "\"");
			for (int i=0; i<protocolList.size(); i++)
				System.out.println(header + "      Participant " + i + ": " + (Participant)protocolList.get(i));
		}
		System.out.println(header + "--------------------------");
	}

	public Iterator<String> getProtocolIterator() {
		// TODO Auto-generated method stub
		return protocolMap.keySet().iterator();
	}

}
