/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wscf.types;

import java.util.ArrayList;

public class ParticipantPairGroup  {
	private ArrayList<String> subProtocols;     //String          {0..n}
	private ArrayList<ParticipantPair> participantPairs; //ParticipantPair {1..n}
	
	public ParticipantPairGroup() {
		this.subProtocols = new ArrayList<String>();
		this.participantPairs = new ArrayList<ParticipantPair>();
	}
	
	public void addSubProtocol(String subProtocol) {
		if(subProtocols==null)
			subProtocols=new ArrayList<String>();
		subProtocols.add(subProtocol);
	}
	
	public void addParticipantPair(ParticipantPair participantPair) {
		if(participantPairs==null)
			participantPairs=new ArrayList<ParticipantPair>();
		participantPairs.add(participantPair);
	}
	
	public void removeSubProtocol(String subProtocol) {
		if (subProtocols!=null)
			subProtocols.remove(subProtocol);
	}
	
	public void removeParticipantPair(ParticipantPair participantPair) {
		if (participantPairs!=null)
			participantPairs.remove(participantPair);
	}
	
	public int getParticipantPairsSize() {
		if (participantPairs!=null)
			return participantPairs.size();
		return 0;
	}
	
	public ParticipantPair getParticipantPair(int i) {
		if (participantPairs!=null)
			return (ParticipantPair)participantPairs.get(i);
		return null;
	}
	
	public String[] getProtocols() {
		return subProtocols.toArray(new String[]{});
	}
}
