/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wscf.types;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;

import org.apache.axis2.addressing.EndpointReference;
import org.oasis.wscaf.core.wscf.exceptions.DuplicateParticipant;
import org.oasis.wscaf.core.wscf.exceptions.ParticipantNotFound;
import org.oasis.wscaf.core.wsctx.types.Context;
import org.oasis.wscaf.core.wsctx.types.ContextIdentifier;
import org.oasis.wscaf.core.wsctx.types.ContextType;

public class RegistrationContext extends Context {

    public static EndpointReference DEFAULT_REGISTRATION_SERVICE = new EndpointReference("http://localhost:8080/axis2/services/WSCAF-RegistrationService");
    
	private EndpointReference registrationService = DEFAULT_REGISTRATION_SERVICE; // {1}
    private ArrayList        subProtocols        = new ArrayList();
	private ParticipantList	 participants        = new ParticipantList(); //Participants grouped by sub-protocols supported
	private Object     		 extension           = null;    //Extension {0..1}

	public void copy(Context context) {
		super.copy(context);
		if (context instanceof RegistrationContext) {
			RegistrationContext regContext = (RegistrationContext) context;
			if (regContext.getRegistrationService() != null) {
				setRegistrationService(regContext.getRegistrationService());
			}
			if (regContext.getSubProtocols() != null) {
				this.subProtocols = regContext.subProtocols;
			}
			if (regContext.participants != null) {
				this.participants = regContext.participants;
			}
			if (regContext.extension != null) {
				this.extension = regContext.extension;
			}
		}
	}
	
	public RegistrationContext (ContextIdentifier contextIdentifier, EndpointReference contextService,
							    EndpointReference contextManager, ContextIdentifier parentContext,
							    Calendar expiresAt, String id, ContextType type) {
		super(contextIdentifier, contextService, contextManager, parentContext,	expiresAt, id, type);
	}

	public RegistrationContext() {
		super();
	}

	public RegistrationContext (ContextIdentifier contextIdentifier, ContextType type,
								EndpointReference registrationService) {
		super(contextIdentifier, type);
		this.registrationService = registrationService;
	}

	public RegistrationContext (ContextIdentifier contextIdentifier, Calendar expiresAt, ContextType type, 
								EndpointReference registrationService) {
		super(contextIdentifier, expiresAt, type);
		this.registrationService = registrationService;
	}

	public RegistrationContext (ContextIdentifier contextIdentifier, ContextIdentifier parentContext,
							    Calendar expiresAt, String id, ContextType type,
							    EndpointReference registrationService) {
		super(contextIdentifier, parentContext, expiresAt, id, type);
		this.registrationService = registrationService;
	}

	public EndpointReference getRegistrationService() {
		return registrationService;
	}

	public void setRegistrationService(EndpointReference registrationService) {
		this.registrationService = registrationService;
	}
	
	public void setExtension (Object extension) {
        this.extension=extension;
	}
	
	public Object getExtension () {
        return extension;
	}
	
	public void addParticipant (Participant participant, String[] protocols) throws DuplicateParticipant {
		participants.addParticipant(participant, protocols);
	}
	
	public void removeParticipant (String protocol, Participant participant) throws ParticipantNotFound {
		participants.removeParticipant(protocol, participant);
	}
	
	public void replaceParticipant (String protocol, ParticipantPair participantPair) {
		participants.replaceParticipant(protocol, participantPair.getOldParticipant(), participantPair.getNewParticipant());
	}
	
	public ParticipantGroup getParticipants (String[] protocols) {
		return participants.getParticipants(protocols);
	}

    public Iterator<String> getParticipantsProtocols() {
        if (participants!=null)
            return participants.getProtocolIterator();
        return null;
    }

	public static RegistrationContext augmentToRegistrationContext(Context context) {
		if (context instanceof RegistrationContext)
			return (RegistrationContext)context;
		
		RegistrationContext registrationContext = new RegistrationContext();
		registrationContext.assign(context);
		return registrationContext;
	}

    public String getSubProtocol(int i) {
        if (subProtocols!=null && i<subProtocols.size())
            return (String)subProtocols.get(i);
        return null;
    }

    public String[] getSubProtocols() {
        if(subProtocols!=null)
            return (String[])subProtocols.toArray(new String[0]);
        return null;
    }

    public void addSubProtocol(String subProtocol) {
        if (subProtocols==null)
            subProtocols = new ArrayList();
        subProtocols.add(subProtocol);
    }
	
    protected void assign(RegistrationContext context) {
        super.assign(context);
        this.registrationService = context.registrationService;
        this.subProtocols        = context.subProtocols;
        this.participants        = context.participants;
        this.extension           = context.extension;
    }

    public void printContents(String header) {
        super.printContents(header);
        if (registrationService!=null)
            System.out.println(header + "Registration Service: "  + registrationService);
        if (subProtocols!=null && subProtocols.size()>0) {
            String subProt="[";
            int i;
            for (i=0; i<subProtocols.size()-1; i++)
                subProt+=subProtocols.get(i) + ", ";
            subProt+=subProtocols.get(i) + "]";
            System.out.println(header + "SubProtocols:         "  + subProt);
        }
        if (participants!=null) {
            System.out.println(header + "Participants:");
            participants.print(header + "    ");
        }
        if (extension!=null)
            System.out.println(header + "Extension:            " + extension.toString());
        System.out.println(header + "--------------------------");
    }

}
