/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wsctx.exceptions;

import javax.xml.namespace.QName;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axiom.soap.SOAPFaultDetail;
import org.apache.axiom.soap.impl.llom.soap11.SOAP11Factory;
import org.apache.axis2.AxisFault;
import org.oasis.wscaf.core.wsctx.types.ContextIdentifier;
import org.oasis.wscaf.utils.Constants;

public class UnknownContext extends AxisFault {
	
    private ContextIdentifier unknownContextId;
    
    public UnknownContext(String cause) {
        super(cause);
        unknownContextId = null;
    }
    
	public UnknownContext(String cause, ContextIdentifier unknownContextId) {
		super(cause);
        this.unknownContextId = unknownContextId;
	}

    public void setUnknownContextId(ContextIdentifier unknownContextId) {
        this.unknownContextId = unknownContextId;
    }
    
    public ContextIdentifier getUnknownContextId() {
        return unknownContextId;
    }

	public QName getFaultCode() {
		return new QName(Constants.CONTEXT_NS,"UnknownContext");
	}
	
	public SOAPFaultDetail getFaultDetailElement() {
		
		OMFactory factory = null;
		SOAPFaultDetail details = super.getFaultDetailElement(); 
		
		if (details == null) {
			factory = new SOAP11Factory();
			details = ((SOAP11Factory)factory).createSOAPFaultDetail();
		}
		else {
			factory = details.getOMFactory();
		}
		
		OMElement ctxIdElement = factory.createOMElement("context-identifier",factory.createOMNamespace(Constants.CONTEXT_NS,"ctx"));
		if (unknownContextId != null) {
			ctxIdElement.setText(unknownContextId.toString());
		}
		
		details.addDetailEntry(ctxIdElement);
		
		return details;
	}
    
}
