/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wsctx.types;

import java.util.ArrayList;
import java.util.Calendar;

import org.apache.axis2.addressing.EndpointReference;
import org.apache.xmlbeans.XmlObject;

public class Context {
	private ContextIdentifier contextIdentifier;
	private EndpointReference  contextService;
	private EndpointReference  contextManager;
	private ContextIdentifier parentContext;
	private Calendar		  expiresAt;
	private String			  id;
	private ArrayList		  augmenters; //Augmenters are stored as objects of class java.lang.Object
	private ContextType		  type;
    
	public void copy(Context context){
		if (context.getContextService() != null)
            setContextService(context.getContextService());
		if (context.getContextManager() != null)
			setContextManager(context.getContextManager());
		if (context.getParentContext() != null)
			setParentContext(context.getParentContext());
		if (context.getExpiresAt() != null) {
			setExpiresAt(context.getExpiresAt());
		}
        if (context.getId()!=null)
            setId(context.getId());
        if (context.getAugmenters() != null) {
        	setAugmenters(context.getAugmenters());
        }
        if (context.getType() != null) {
        	setType(context.getType());
        }
	}
	public Context (ContextIdentifier contextIdentifier, EndpointReference contextService,
			EndpointReference contextManager, ContextIdentifier parentContext,
					Calendar expiresAt, String id, ContextType type) {
		
		this.contextIdentifier = contextIdentifier;
		this.contextService	   = contextService;
		this.contextManager	   = contextManager;
		this.parentContext	   = parentContext;
		this.expiresAt		   = expiresAt;
		this.id				   = id;
		this.type			   = type;
		this.augmenters		   = new ArrayList();
	}
	
	public Context() {
		this(null, null, null, null, null, null, null);
	}
	
	public Context(ContextIdentifier contextIdentifier, ContextType type) {
		this(contextIdentifier, null, null, null, null, null, type);
	}

	public Context(ContextIdentifier contextIdentifier, Calendar expiresAt, ContextType type) {
		this(contextIdentifier, null, null, null, expiresAt, null, type);
	}

	public Context(ContextIdentifier contextIdentifier, ContextIdentifier parentContext, Calendar expiresAt, String id, ContextType type) {
		this(contextIdentifier, null, null, parentContext, expiresAt, id, type);
	}

	public final ContextIdentifier getContextIdentifier() {
		return contextIdentifier;
	}
	public final void setContextIdentifier(ContextIdentifier contextIdentifier) {
		this.contextIdentifier = contextIdentifier;
	}
	
	public final EndpointReference getContextManager() {
		return contextManager;
	}
	public final void setContextManager(EndpointReference contextManager) {
		this.contextManager = contextManager;
	}
	
	public final EndpointReference getContextService() {
		return contextService;
	}
	public final void setContextService(EndpointReference contextService) {
		this.contextService = contextService;
	}
	
	public final Calendar getExpiresAt() {
		return expiresAt;
	}
	public final void setExpiresAt(Calendar expiresAt) {
		this.expiresAt = expiresAt;
	}
	
	public final String getId() {
		return id;
	}
	public final void setId(String id) {
		this.id = id;
	}
	
	public final ContextIdentifier getParentContext() {
		return parentContext;
	}
	public final void setParentContext(ContextIdentifier parentContext) {
		this.parentContext = parentContext;
	}
	
	public final ArrayList getAugmenters() {
		return augmenters;
	}
	public final void setAugmenters(ArrayList augmenters) {
		this.augmenters = augmenters;
	}

	public final ContextType getType() {
		return type;
	}

	public final void setType(ContextType type) {
		this.type = type;
	}

    public void printContents() {
        printContents("");
    }
	
	public void printContents(String header) {
		System.out.println(header + "---- Printing Context ----");
		System.out.println(header + "Context Identifier:  \"" + contextIdentifier + "\"");
		if (contextService!=null)
			System.out.println(header + "Context Service:      "  + contextService);
		if (contextManager!=null)
			System.out.println(header + "Context Manager:      "  + contextManager);
		if (parentContext!=null)
			System.out.println(header + "Parent Context Id:   \"" + parentContext + "\"");
		if (expiresAt!=null)
		    System.out.println(header + "ExpiresAt             " + expiresAt);
		if (id!=null)
			System.out.println(header + "Id:                  \"" + id + "\"");
		if (type!=null)
			System.out.println(header + "Context Type          " + type);
        if (augmenters!=null && augmenters.size()>0) {
            System.out.println(header + "Augmenters: " + augmenters.size() + " elements");
            for (int i=0; i<augmenters.size(); i++) {
                System.out.println(header + "   Augmenter #" + i + ":");
                System.out.println(header + "      " + augmenters.get(i).toString());
            }
        }
		System.out.println(header + "--------------------------");
	}
	
	protected void assign(Context context) {
		this.contextIdentifier = context.contextIdentifier;
		this.contextService	   = context.contextService;
		this.contextManager	   = context.contextManager;
		this.parentContext	   = context.parentContext;
		this.expiresAt		   = context.expiresAt;
		this.id				   = context.id;
		this.augmenters		   = context.augmenters;
		this.type			   = context.type;
	}
    
    public int augmentersSize() {
        if (augmenters==null)
            return 0;
        return augmenters.size();
    }
    
    public void addAugmenter(XmlObject augmenter) {
        augmenters.add(augmenter);
    }
    
    public Object getAugmenter(int i) {
        if(augmenters==null)
            return null;
        return augmenters.get(i);
    }
    
}
