/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wsctx.types;

import javax.xml.namespace.QName;

import org.oasis.wscaf.core.wslra.types.LRAState;
import org.oasis.wscaf.utils.Constants;

public class State {

	protected QName state;
	public static QName STATE_RUNNING   = new QName(Constants.CONTEXT_NS,"activity.status.wsctx.Running");
	public static QName STATE_COMPLETED = new QName(Constants.CONTEXT_NS,"activity.status.wsctx.Completed");
	public static QName STATE_TIMED_OUT = new QName(Constants.CONTEXT_NS,"activity.status.wsctx.TimedOut");
	
	
	
	public State() {
		state=State.STATE_RUNNING;
	}

	public State(QName state) {
		if (isSupported(state)) {
			this.state = state;
		}
	}
	
	public boolean isSupported(QName stateName) {
		return ((stateName.equals(State.STATE_RUNNING)) || 
				(stateName.equals(State.STATE_COMPLETED)) || 
				(stateName.equals(State.STATE_TIMED_OUT)));
	}
	
	public final void setState(QName state) {
		this.state = state;
	}
	
	public QName getState() {
		return this.state;
	}
	
	public final String toString() {
		return state.toString();
	}
	/*
	public boolean isValid() {
		return state.equals(STATE_TIMED_OUT) ||
			   state.equals(STATE_COMPLETED) ||
			   state.equals(STATE_RUNNING);
	}
	*/
	public void setCompleted() {
		state=State.STATE_COMPLETED;
	}
	public final void setTimedOut() {
		state=State.STATE_TIMED_OUT;
	}
	
	public final boolean isTimedOut() {
		return state.equals(State.STATE_TIMED_OUT);
	}
	
	public boolean isCompleted() {
		return state.equals(State.STATE_COMPLETED)       || 
               state.equals(LRAState.STATE_LRA_COMPENSATED) ||
               state.equals(LRAState.STATE_LRA_COMPLETED);
	}
    
	public final boolean isRunning() {
        return state.equals(State.STATE_RUNNING);
    }
	
	
}
 