/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.core.wslra.types;

import javax.xml.namespace.QName;

import org.oasis.wscaf.core.wsctx.types.State;
import org.oasis.wscaf.utils.Constants;

public class LRAState extends State {

	public static QName STATE_LRA_COMPENSATING       = new QName(Constants.LRA_NS,"activity.status.tx-lra.Compensating");
	public static QName STATE_LRA_COMPENSATED        = new QName(Constants.LRA_NS,"activity.status.tx-lra.Compensated");
	public static QName STATE_LRA_FAILED_TO_COMPENSATE = new QName(Constants.LRA_NS,"activity.status.tx-lra.FailedToCompensate");
	
	public static QName STATE_LRA_COMPLETING         = new QName(Constants.LRA_NS,"activity.status.tx-lra.Completing");
	public static QName STATE_LRA_COMPLETED          = new QName(Constants.LRA_NS,"activity.status.tx-lra.Completed");
	public static QName STATE_LRA_FAILED_TO_COMPLETE = new QName(Constants.LRA_NS,"activity.status.tx-lra.FailedToComplete");
	
	public static QName STATE_LRA_FORGETING         = new QName(Constants.LRA_NS,"activity.status.tx-lra.Forgeting");
	public static QName STATE_LRA_FORGOT          = new QName(Constants.LRA_NS,"activity.status.tx-lra.Forgot");
	public static QName STATE_LRA_FAILED_TO_FORGOT = new QName(Constants.LRA_NS,"activity.status.tx-lra.FailedToForgot");

	public LRAState() {
		super();
		// TODO Auto-generated constructor stub
	}

	public LRAState(QName state) {
		if ((state.equals(LRAState.STATE_LRA_COMPENSATING)) || 
			(state.equals(LRAState.STATE_LRA_COMPLETING)) ||
			(state.equals(LRAState.STATE_LRA_FORGETING))) {
			this.state = state;
		}
	}
	
	public QName complete(boolean success) {
		
		if (state.equals(LRAState.STATE_LRA_COMPENSATING)) {
			if (success) {
				this.state = LRAState.STATE_LRA_COMPENSATED;
			}
			else {
				this.state = LRAState.STATE_LRA_FAILED_TO_COMPENSATE;
			}
		}
		
		if (state.equals(LRAState.STATE_LRA_COMPLETING)) {
			if (success) {
				this.state = LRAState.STATE_LRA_COMPLETED;
			}
			else {
				this.state = LRAState.STATE_LRA_FAILED_TO_COMPLETE;
			}
		}
		
		if (state.equals(LRAState.STATE_LRA_FORGETING)) {
			if (success) {
				this.state = LRAState.STATE_LRA_FORGOT;
			}
			else {
				this.state = LRAState.STATE_LRA_FAILED_TO_FORGOT;
			}
		}
		
		return this.state;
	}
	
	public boolean equals(LRAState state) {
		return this.state.equals(state.getState());
	}
	
	public boolean equals(QName state) {
		return this.state.equals(state);
	}
}
