/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.middleware.services;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.xmlbeans.XmlException;
import org.oasis.wscaf.core.wsctx.exceptions.InvalidContextStructure;
import org.oasis.wscaf.core.wsctx.exceptions.NoContext;
import org.oasis.wscaf.core.wsctx.services.ContextManager;
import org.oasis.wscaf.core.wsctx.types.Context;
import org.oasis.wscaf.core.wsctx.types.ContextIdentifier;
import org.oasis.wscaf.module.utils.ContextModuleConstants;
import org.oasis.wscaf.types.wsctx.ContentsDocument;
import org.oasis.wscaf.types.wsctx.ContentsSetDocument;
import org.oasis.wscaf.types.wsctx.ContextType;
import org.oasis.wscaf.types.wsctx.GetContentsDocument;
import org.oasis.wscaf.types.wsctx.SetContentsDocument;
import org.oasis.wscaf.types.wsctx.ContentsDocument.Contents;
import org.oasis.wscaf.utils.SOAPUtils;

public class ContextManagerSkeleton implements ContextManagerSkeletonInterface {

	public ContentsDocument getContents(GetContentsDocument getContents) throws AxisFault {
		System.out.println("[ContextManager][GetContents] Request received");
		
		org.oasis.wscaf.bindings.wsctx.ContextType ctx = (org.oasis.wscaf.bindings.wsctx.ContextType) MessageContext.getCurrentMessageContext().getProperty(ContextModuleConstants.CurrentContextProperty);

		ContextIdentifier ctxId = null;
		if (ctx==null) {
			throw new NoContext("No context could be found in header");
		}
		else {
			ctxId = new ContextIdentifier(ctx.getContextIdentifier().getValue());
		}
		
		ContextManager contextManager = ContextManager.getInstance();

		//System.out.println("[ContextManager][GetContents] Invoking business logic");
		Context context = contextManager.getContents(ctxId);
		    
		ContentsDocument contentsDocument = ContentsDocument.Factory.newInstance();
		Contents contents = contentsDocument.addNewContents();
		
		try {
			contents.setContext(SOAPUtils.translateContext(context));
		} catch (XmlException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		    
		System.out.println("Contents Document:\n" + contentsDocument.toString());
		
		MessageContext outMessageContext = MessageContext.getCurrentMessageContext().getOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
		outMessageContext.setProperty(ContextModuleConstants.OutContextProperty, ctx);
		
		System.out.println("[ContextManager][GetContents] Ending request");
		return contentsDocument;
	}
	

	public ContentsSetDocument setContents(SetContentsDocument setContents) throws AxisFault {
		
		System.out.println("[ContextManager][SetContents] Request received");
		
		org.oasis.wscaf.bindings.wsctx.ContextType ctx = (org.oasis.wscaf.bindings.wsctx.ContextType) MessageContext.getCurrentMessageContext().getProperty(ContextModuleConstants.CurrentContextProperty);

		ContextIdentifier ctxId = null;
		if (ctx==null) {
			throw new NoContext("No context could be found in header");
		}
		else {
			ctxId = new ContextIdentifier(ctx.getContextIdentifier().getValue());
		}
		
		ContextManager contextManager = ContextManager.getInstance();

		ContextType newContext = setContents.getSetContents().getContext();
		newContext.getContextIdentifier().setStringValue(ctxId.toString());
		
		//System.out.println("[ContextManager][SetContents] Invoking business logic");
		try {
			contextManager.setContents(SOAPUtils.translateContext(newContext, null));
		} catch (Exception e) {
			e.printStackTrace();
			throw new InvalidContextStructure("The incomming contents is not a valid context: "+e.getMessage());
		}
		
		ContentsSetDocument contentsSetDocument = ContentsSetDocument.Factory.newInstance();
		contentsSetDocument.addNewContentsSet();
		
		MessageContext outMessageContext = MessageContext.getCurrentMessageContext().getOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
		outMessageContext.setProperty(ContextModuleConstants.OutContextProperty, ctx);
		
		System.out.println("[ContextManager][SetContents] Ending request");
		
		return contentsSetDocument;
	}

}
