/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.middleware.services;

import java.math.BigDecimal;
import java.util.Calendar;

import javax.xml.namespace.QName;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.xmlbeans.GDuration;
import org.apache.xmlbeans.XmlCursor;
import org.apache.xmlbeans.XmlException;
import org.oasis.wscaf.bindings.wsctx.ContextIdentifierType;
import org.oasis.wscaf.bindings.wsctx.ObjectFactory;
import org.oasis.wscaf.core.wsctx.exceptions.NoContext;
import org.oasis.wscaf.core.wsctx.services.ContextService;
import org.oasis.wscaf.core.wsctx.types.Context;
import org.oasis.wscaf.core.wsctx.types.ContextIdentifier;
import org.oasis.wscaf.core.wsctx.types.ContextType;
import org.oasis.wscaf.core.wsctx.types.Duration;
import org.oasis.wscaf.module.utils.ContextModuleConstants;
import org.oasis.wscaf.types.wsctx.AssertionType;
import org.oasis.wscaf.types.wsctx.BeginDocument;
import org.oasis.wscaf.types.wsctx.BegunDocument;
import org.oasis.wscaf.types.wsctx.CompleteDocument;
import org.oasis.wscaf.types.wsctx.CompletedDocument;
import org.oasis.wscaf.types.wsctx.ContextDocument;
import org.oasis.wscaf.types.wsctx.GetStatusDocument;
import org.oasis.wscaf.types.wsctx.GetTimeoutDocument;
import org.oasis.wscaf.types.wsctx.GotStatusDocument;
import org.oasis.wscaf.types.wsctx.SetTimeoutDocument;
import org.oasis.wscaf.types.wsctx.TimeoutDocument;
import org.oasis.wscaf.types.wsctx.TimeoutSetDocument;
import org.oasis.wscaf.types.wsctx.GotStatusDocument.GotStatus;
import org.oasis.wscaf.types.wsctx.TimeoutDocument.Timeout;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.SOAPUtils;

/**
 * ContextServiceSkeleton java skeleton for the axisService
 */
public class ContextServiceSkeleton implements ContextServiceSkeletonInterface {

	/**
	 * Auto generated method signature
	 * 
	 * @param begin3
	 * @throws AxisFault 
	 * 
	 */

	public BegunDocument begin(BeginDocument beginRequest) throws AxisFault {
		System.out.println("[ContextService][Begin] Request received");
		
		org.oasis.wscaf.bindings.wsctx.ContextType ctx = (org.oasis.wscaf.bindings.wsctx.ContextType) MessageContext.getCurrentMessageContext().getProperty(ContextModuleConstants.CurrentContextProperty);
		
		String ctxId = null;
		if (ctx != null) {
			ctxId = ctx.getContextIdentifier().getValue();
		}
		
		ContextIdentifier parentContext = null;
		if (ctxId != null) {
			parentContext = new ContextIdentifier(ctxId);
		}
		
		ContextService contextService = ContextService.getInstance();
		
		Calendar expiresAt = (Calendar) beginRequest.getBegin().getExpiresAt();
		String id = beginRequest.getBegin().getId();
		// TODO Handle type
		//System.out.println("[ContextService][Begin] Getting type");
		//ContextType type = new ContextType(beginDocument.getBegin().getType().toString());
		ContextType type = new ContextType();
			
		//System.out.println("[ContextService][Begin] Invoking business logic");
		Context context = contextService.begin(expiresAt, id, parentContext, type);

		BegunDocument begunDocument = BegunDocument.Factory.newInstance();
		AssertionType begun = begunDocument.addNewBegun();
		
		/* 
		 * Add context identifier to begun response. 
		 * Temporary solution while we update to Axis 1.2 so we can take headers from stub
		 */ 
		XmlCursor cursor = begun.newCursor();
		cursor.toFirstContentToken();
		cursor.beginElement(Constants.CONTEXT_IDENTIFIER_NAME);
		cursor.insertChars(context.getContextIdentifier().toString());
		
		ContextDocument contextDocument = ContextDocument.Factory.newInstance();
		System.out.println("[ContextServiceMessageReceiverInOutHelper] Adding context id: "
							+ context.getContextIdentifier().toString());
		try {
			contextDocument.setContext(SOAPUtils.translateContext(context));
		} catch (XmlException e) {
			e.printStackTrace();
			throw new AxisFault("Could not generate new context",e);
		}
			
		//MessageContext.getCurrentMessageContext().setProperty(Constants.NewContextProperty, omContext);
		MessageContext outMessageContext = MessageContext.getCurrentMessageContext().getOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
		
		ObjectFactory factory = new ObjectFactory();
		ContextIdentifierType newCtxId = factory.createContextIdentifierType();
		newCtxId.setValue(context.getContextIdentifier().toString());
		org.oasis.wscaf.bindings.wsctx.ContextType newCtx = factory.createContextType();
		newCtx.setContextIdentifier(newCtxId);
		
		outMessageContext.setProperty(ContextModuleConstants.OutContextProperty, newCtx);
		
		System.out.println("[ContextService][Begin] Ending request");

		return begunDocument;
	}

	/**
	 * Auto generated method signature
	 * 
	 * @param complete5
	 * @throws AxisFault 
	 * 
	 */

	public CompletedDocument complete(CompleteDocument completeRequest) throws AxisFault {
		System.out.println("[ContextService][Complete] Request received");
		
		org.oasis.wscaf.bindings.wsctx.ContextType ctx = (org.oasis.wscaf.bindings.wsctx.ContextType) MessageContext.getCurrentMessageContext().getProperty(ContextModuleConstants.CurrentContextProperty);

		ContextIdentifier context = null;
		if (ctx==null) {
			throw new NoContext("No context could be found in header");
		}
		else {
			context = new ContextIdentifier(ctx.getContextIdentifier().getValue());
		}
		
		//ContextIdentifier context = new ContextIdentifier((String) MessageContext.getCurrentMessageContext().getProperty(Constants.CurrentContextIdentifier));
				
		ContextService contextService = ContextService.getInstance();

		String status = SOAPUtils.getAnyElementValue(completeRequest.getComplete(), Constants.COMPLETE_STATUS_NAME); 

		//System.out.println("[ContextService][Complete] Invoking business logic");
		contextService.complete(context, status);
		
		CompletedDocument completedDocument = CompletedDocument.Factory.newInstance();
		completedDocument.addNewCompleted();
		/*XmlCursor cursor = completedDocument.addNewCompleted().newCursor(); 
		cursor.toFirstContentToken();
		StatusDocument currentStatus = StatusDocument.Factory.newInstance();
		currentStatus.setStatus(new QName(contextService.getStatus(context)));
		currentStatus.newCursor().copyXml(cursor);*/
		
		//MessageContext.getCurrentMessageContext().setProperty(Constants.CurrentContextProperty, context);
		MessageContext outMessageContext = MessageContext.getCurrentMessageContext().getOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
		outMessageContext.setProperty(ContextModuleConstants.OutContextProperty, ctx);
		
		System.out.println("[ContextService][Complete] Ending request");
		
		return completedDocument;
		
	}

	/**
	 * Auto generated method signature
	 * 
	 * @param getStatus8
	 * @throws AxisFault 
	 * 
	 */

	public GotStatusDocument getStatus(GetStatusDocument getStatusRequest) throws AxisFault {
		System.out.println("[ContextService][GetStatus] Request received");
		
		org.oasis.wscaf.bindings.wsctx.ContextType ctx = (org.oasis.wscaf.bindings.wsctx.ContextType) MessageContext.getCurrentMessageContext().getProperty(ContextModuleConstants.CurrentContextProperty);
		
		if (ctx == null) {
			throw new NoContext("No context found in Header");
		}
		
		ContextIdentifier context = new ContextIdentifier(ctx.getContextIdentifier().getValue());
		
		ContextService contextService = ContextService.getInstance();

		//System.out.println("[ContextService][GetStatus] Invoking business logic");
		QName status = contextService.getStatus(context);
		
		GotStatusDocument gotStatusDocument = GotStatusDocument.Factory.newInstance();
		GotStatus gotStatus = gotStatusDocument.addNewGotStatus();
		if (status==null)
			gotStatus.setNil();
		else
			gotStatus.setStatus(status);
		
		//MessageContext.getCurrentMessageContext().setProperty(Constants.CurrentContextProperty, context);
		MessageContext outMessageContext = MessageContext.getCurrentMessageContext().getOperationContext().getMessageContext(WSDLConstants.MESSAGE_LABEL_OUT_VALUE);
		outMessageContext.setProperty(ContextModuleConstants.OutContextProperty, ctx);
		
		System.out.println("[ContextService][GetStatus] Ending request");
		
		return gotStatusDocument;
	}

	/**
	 * Auto generated method signature
	 * 
	 * @param setTimeout11
	 * @throws AxisFault
	 * 
	 */

	public TimeoutSetDocument setTimeout(SetTimeoutDocument setTimeoutRequest) throws AxisFault {
		System.out.println("[ContextService][SetTimeout] Request received");
		
		ContextService contextService = ContextService.getInstance();
		
		GDuration gTimeout = setTimeoutRequest.getSetTimeout().getTimeout();
		Duration timeout = new Duration(gTimeout.getYear(),
		                                gTimeout.getMonth(),
		                                gTimeout.getDay(),
		                                gTimeout.getHour(),
		                                gTimeout.getMinute(), 
		                                gTimeout.getSecond(),
		                                gTimeout.getFraction().multiply(new BigDecimal(1000)).intValue());
		
		//System.out.println("[ContextService][SetTimeout] Invoking business logic");
		contextService.setTimeout(timeout);

		TimeoutSetDocument timeoutSetDocument = TimeoutSetDocument.Factory.newInstance();
		timeoutSetDocument.addNewTimeoutSet().setTimeout(gTimeout);
		
		
		System.out.println("[ContextService][SetTimeout] Ending request");
		
		return timeoutSetDocument;
	}

	/**
	 * Auto generated method signature
	 * 
	 * @param getTimeout14
	 * @throws AxisFault
	 * 
	 */

	public TimeoutDocument getTimeout(GetTimeoutDocument getTimeoutRequest) throws AxisFault {
		System.out.println("[ContextService][GetTimeout] Request received");
		ContextService contextService = ContextService.getInstance();

		//System.out.println("[ContextService][GetTimeout] Invoking business logic");
		Duration timeout = contextService.getTimeout();
				   
		TimeoutDocument getTimeoutDocument = TimeoutDocument.Factory.newInstance();
		Timeout time = getTimeoutDocument.addNewTimeout();
		
		if (timeout!=null) {
			GDuration duration = new GDuration(1,
					timeout.getYears(),
					timeout.getMonths(),
					timeout.getDays(),
					timeout.getHours(),
					timeout.getMinutes(),
					timeout.getSeconds(),
					(new BigDecimal(timeout.getMilliseconds())).divide(new BigDecimal(1000), BigDecimal.ROUND_UNNECESSARY));
			
		        	time.setTimeout(duration);
		}
		
		System.out.println("[ContextService][GetTimeout] Ending request");
		
		return getTimeoutDocument;
	}

}
