/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.tests;

import java.rmi.RemoteException;

import javax.xml.stream.XMLStreamException;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.oasis.wscaf.middleware.services.ContextServiceStub;
import org.oasis.wscaf.types.wsctx.BeginDocument;
import org.oasis.wscaf.types.wsctx.CompleteDocument;
import org.oasis.wscaf.types.wsctx.ContextDocument;
import org.oasis.wscaf.types.wsctx.ContextType;
import org.oasis.wscaf.types.wsctx.GetStatusDocument;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.ContextUtils;
import org.oasis.wscaf.utils.ConversionUtils;

public class ContextServiceTest {

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		try {
			
			EndpointReference ctxServiceRef = new EndpointReference("http://localhost:8081/axis2/services/ContextService");
			
			ContextServiceStub stub = new ContextServiceStub(ctxServiceRef.getAddress());
			//stub._getServiceClient().setCachingOperationContext(true);
			
			//Begin first context
			BeginDocument beginDocument = BeginDocument.Factory.newInstance();
			beginDocument.addNewBegin();
			stub.begin(beginDocument);
			
			String newId = null;
			OperationContext opContext = stub._getServiceClient().getLastOperationContext();
			if (opContext != null) {
				MessageContext messageContext = opContext.getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
				System.out.println("IN Message contents: \n"
						+ messageContext.getEnvelope().getHeader());
				ContextUtils.getContext(messageContext, Constants.CONTEXT_NAME);
				newId = (String) messageContext.getProperty(Constants.CurrentContextIdentifier);
			}
			else {
				throw new Exception("Cannot get operation context");
			}
			//Begin nested context
			ContextDocument ctx = ContextDocument.Factory.newInstance();
			ContextType context = ctx.addNewContext();
			context.addNewContextIdentifier().setStringValue(newId);
			
			stub._getServiceClient().addHeader(ConversionUtils.xBeanToOM(ctx));
			
			stub.begin(beginDocument);
			
			stub._getServiceClient().removeHeaders();
			
			//Get status before completion
			GetStatusDocument getStatusDocument = GetStatusDocument.Factory.newInstance();
			getStatusDocument.addNewGetStatus();
			
			stub.getStatus(getStatusDocument, ctx);
			
			//Complete context
			CompleteDocument completeDocument = CompleteDocument.Factory.newInstance();
			completeDocument.addNewComplete();
			
			stub.complete(completeDocument, ctx);
			
			//Get status after completion
			stub.getStatus(getStatusDocument, ctx);
			
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XMLStreamException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
		
	}

}
