/**
  * JASS Project. WS-CAF.
  *
  * Copyright (c) 2006, Distributed Systems Lab, Universidad Politecnica 
  * de Madrid (Spain)
  * All rights reserved.
  * Contact:
  *        web page: http://lsd.ls.fi.upm.es/lsd
  *        e-mail: mpatino@fi.upm.es or rjimenez@fi.upm.es
  *
  * Redistribution and use in source and binary forms, with or without 
  * modification, are permitted provided that the following conditions are met:
  *
  *    - Redistributions of source code must retain the above copyright 
  *      notice, this list of conditions and the following disclaimer.
  *    - Redistributions in binary form must reproduce the above copyright 
  *      notice, this list of conditions and the following disclaimer in the 
  *      documentation and/or other materials provided with the distribution.
  *    - Neither the name of the Distributed Systems Lab nor the names of 
  *      its contributors may be used to endorse or promote products derived from 
  *      this software without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
  * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
  * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
  * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
  * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
  * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, *
  * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
  * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
  * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
  * Contributors: 
  * Francisco Pérez-Sorrosal (fpsorrosal@zipi.fi.upm.es)
  * Rubén Naranjo Izquierdo (rnaranjo@alumnos.upm.es)
  * José Antonio Sánchez Murillo (jsanchez@alumnos.upm.es)
  * 
  **/

package org.oasis.wscaf.tests;

import java.rmi.RemoteException;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.OperationContext;
import org.apache.axis2.wsdl.WSDLConstants;
import org.apache.xmlbeans.XmlException;
import org.oasis.wscaf.middleware.services.ContextManagerStub;
import org.oasis.wscaf.middleware.services.ContextServiceStub;
import org.oasis.wscaf.middleware.services.RegistrationServiceStub;
import org.oasis.wscaf.types.wscf.AddParticipantDocument;
import org.oasis.wscaf.types.wscf.RegistrationContextDocument;
import org.oasis.wscaf.types.wscf.RemoveParticipantDocument;
import org.oasis.wscaf.types.wscf.ReplaceParticipantDocument;
import org.oasis.wscaf.types.wsctx.BeginDocument;
import org.oasis.wscaf.types.wsctx.CompleteDocument;
import org.oasis.wscaf.types.wsctx.ContextDocument;
import org.oasis.wscaf.types.wsctx.GetContentsDocument;
import org.oasis.wscaf.types.wsctx.GetStatusDocument;
import org.oasis.wscaf.utils.ClientUtils;
import org.oasis.wscaf.utils.Constants;
import org.oasis.wscaf.utils.ContextUtils;
import org.oasis.wscaf.utils.ClientUtils.ParticipantsType;
import org.oasis.wscaf.utils.ClientUtils.ReplaceParticipantType;
import org.oasis.wscaf.utils.exceptions.ElementNotFound;

public class RegistrationServiceTest {
	
	public static void main(String[] args) {
		try {
			ConfigurationContext cc = ConfigurationContextFactory.createConfigurationContextFromFileSystem(null, null);
			
			EndpointReference ctxServiceRef = new EndpointReference("http://localhost:8081/axis2/services/ContextService");
			EndpointReference ctxManagerRef = new EndpointReference("http://localhost:8081/axis2/services/ContextManager");
			EndpointReference regServiceRef = new EndpointReference("http://localhost:8081/axis2/services/RegistrationService");
			
			ContextServiceStub stubSrvc = new ContextServiceStub(cc,ctxServiceRef.getAddress());
			//stubSrvc._getServiceClient().setCachingOperationContext(true);
			//stubSrvc._getServiceClient().engageModule(new QName(org.apache.axis2.Constants.MODULE_ADDRESSING));
			
			stubSrvc._getServiceClient().setTargetEPR(ctxServiceRef);
			stubSrvc._getServiceClient().getOptions().setTo(ctxServiceRef);
			
			//Begin first context
			BeginDocument beginDocument = BeginDocument.Factory.newInstance();
			beginDocument.addNewBegin();
			stubSrvc.begin(beginDocument);
			
			String newId = null;
			OperationContext opContext = stubSrvc._getServiceClient().getLastOperationContext();
			if (opContext != null) {
				MessageContext messageContext = opContext.getMessageContext(WSDLConstants.MESSAGE_LABEL_IN_VALUE);
				System.out.println("IN Message contents: \n"
						+ messageContext.getEnvelope().getHeader());
				ContextUtils.getContext(messageContext, Constants.CONTEXT_NAME);
				newId = (String) messageContext.getProperty(Constants.CurrentContextIdentifier);
			}
			else {
				throw new Exception("Cannot get operation context");
			}
			
			//Add participants
			RegistrationContextDocument regCtxDoc = RegistrationContextDocument.Factory.newInstance();
			regCtxDoc.addNewRegistrationContext().addNewContextIdentifier().setStringValue(newId);
			
			ContextDocument ctxDoc = ContextDocument.Factory.newInstance();
			ctxDoc.addNewContext().addNewContextIdentifier().setStringValue(newId);
			
			EndpointReference eprParticipant1 = new EndpointReference("http://localhost:8081/axis2/services/TestService1");
			EndpointReference eprParticipant2 = new EndpointReference("http://localhost:8081/axis2/services/TestService2");
			EndpointReference eprParticipant3 = new EndpointReference("http://localhost:8081/axis2/services/TestService3");
			EndpointReference eprParticipant4 = new EndpointReference("http://localhost:8081/axis2/services/TestService4");
			
			ParticipantsType lraParticipants = new ParticipantsType("LRA",new EndpointReference[]{eprParticipant1,eprParticipant2});
			ParticipantsType acidParticipants = new ParticipantsType("ACID",new EndpointReference[]{eprParticipant3,eprParticipant4});
			
			AddParticipantDocument addParticipantDoc = ClientUtils.createAddParticipant(new ParticipantsType[]{lraParticipants,acidParticipants});
			
			RegistrationServiceStub stubReg = new RegistrationServiceStub(cc,regServiceRef.getAddress());
			//stubReg._getServiceClient().engageModule(new QName(org.apache.axis2.Constants.MODULE_ADDRESSING));
			
			
			stubReg._getServiceClient().setTargetEPR(regServiceRef);
			stubReg._getServiceClient().getOptions().setTo(regServiceRef);
			
			stubReg.addParticipant(addParticipantDoc, regCtxDoc);
			
			//Get Content
			ContextManagerStub stubMan = new ContextManagerStub(cc,ctxManagerRef.getAddress());
			//stubMan._getServiceClient().engageModule(new QName(org.apache.axis2.Constants.MODULE_ADDRESSING));
			
			
			stubMan._getServiceClient().setTargetEPR(ctxManagerRef);
			stubMan._getServiceClient().getOptions().setTo(ctxManagerRef);
			
			GetContentsDocument getContentsDoc = GetContentsDocument.Factory.newInstance();
			getContentsDoc.addNewGetContents();
			
			stubMan.getContents(getContentsDoc, ctxDoc);
			
			//Replace participants
			ReplaceParticipantType replace1 = new ReplaceParticipantType("ACID",eprParticipant3,eprParticipant1);
			ReplaceParticipantType replace2 = new ReplaceParticipantType("ACID",eprParticipant4,eprParticipant2);
			
			ReplaceParticipantType[] replaces = new ReplaceParticipantType[]{replace1,replace2};
			ReplaceParticipantDocument replace = ClientUtils.createReplaceParticipant(replaces);
			
			stubReg.replaceParticipant(replace, regCtxDoc);
			
			//Get Contents
			stubMan.getContents(getContentsDoc, ctxDoc);
			
			//Remove Participants
			ParticipantsType removes = new ParticipantsType("ACID",new EndpointReference[]{eprParticipant1});
			ParticipantsType[] removePrticipants = new ParticipantsType[]{removes};
			RemoveParticipantDocument removeDoc = ClientUtils.createRemoveParticipant(removePrticipants);
			
			stubReg.removeParticipant(removeDoc, regCtxDoc);
			
			//Get Contents
			stubMan.getContents(getContentsDoc, ctxDoc);
			
			//Get Status
			GetStatusDocument getStatusDoc = GetStatusDocument.Factory.newInstance();
			getStatusDoc.addNewGetStatus();
			stubSrvc.getStatus(getStatusDoc, ctxDoc);
			
			//Complete context
			CompleteDocument completeDoc = ClientUtils.createCompleteDocument("FAIL");
			stubSrvc.complete(completeDoc, ctxDoc);
			
			//Get Status
			stubSrvc.getStatus(getStatusDoc, ctxDoc);
			
		} catch (AxisFault e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (RemoteException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (XmlException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ElementNotFound e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} 
	}

}
